"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.attribute = void 0;
const protocols_1 = require("./protocols");
const binary_set_1 = require("./binary-set");
const number_value_set_1 = require("./number-value-set");
require("reflect-metadata");
function attribute(parameters = {}) {
    return (target, propertyKey) => {
        if (!Object.prototype.hasOwnProperty.call(target, protocols_1.DocumentSchema)) {
            Object.defineProperty(target, protocols_1.DocumentSchema, { value: deriveBaseSchema(target) });
        }
        target[protocols_1.DocumentSchema][propertyKey] = metadataToSchemaType(Reflect.getMetadata(protocols_1.METADATA_TYPE_KEY, target, propertyKey), parameters);
    };
}
exports.attribute = attribute;
function metadataToSchemaType(ctor, declaration) {
    let { type } = declaration, rest = __rest(declaration, ["type"]);
    if (type === undefined) {
        if (ctor) {
            if (ctor === String) {
                type = 'String';
            }
            else if (ctor === Number) {
                type = 'Number';
            }
            else if (ctor === Boolean) {
                type = 'Boolean';
            }
            else if (ctor === Date || ctor.prototype instanceof Date) {
                type = 'Date';
            }
            else if (ctor === binary_set_1.BinarySet ||
                ctor.prototype instanceof binary_set_1.BinarySet) {
                type = 'Set';
                rest.memberType = 'Binary';
            }
            else if (ctor === number_value_set_1.NumberValueSet ||
                ctor.prototype instanceof number_value_set_1.NumberValueSet) {
                type = 'Set';
                rest.memberType = 'Number';
            }
            else if (ctor === Set || ctor.prototype instanceof Set) {
                type = 'Set';
                if (!('memberType' in rest)) {
                    throw new Error('Invalid set declaration. You must specify a memberType');
                }
            }
            else if (ctor === Map || ctor.prototype instanceof Map) {
                type = 'Map';
                if (!('memberType' in rest)) {
                    throw new Error('Invalid map declaration. You must specify a memberType');
                }
            }
            else if (ctor.prototype[protocols_1.DocumentSchema]) {
                type = 'Document';
                rest.members = ctor.prototype[protocols_1.DocumentSchema];
                rest.valueConstructor = ctor;
            }
            else if (isBinaryType(ctor)) {
                type = 'Binary';
            }
            else if (ctor === Array || ctor.prototype instanceof Array) {
                if ('members' in declaration) {
                    type = 'Tuple';
                }
                else if ('memberType' in declaration) {
                    type = 'List';
                }
                else {
                    type = 'Collection';
                }
            }
            else {
                type = 'Any';
            }
        }
        else {
            type = 'Any';
        }
    }
    return Object.assign(Object.assign({}, rest), { type });
}
function deriveBaseSchema(target) {
    if (target && typeof target === 'object') {
        const prototype = Object.getPrototypeOf(target);
        if (prototype) {
            return Object.assign(Object.assign({}, deriveBaseSchema(prototype)), Object.prototype.hasOwnProperty.call(prototype, protocols_1.DocumentSchema)
                ? prototype[protocols_1.DocumentSchema]
                : {});
        }
    }
    return {};
}
function isBinaryType(arg) {
    return arg === Uint8Array || arg.prototype instanceof Uint8Array ||
        arg === Uint8ClampedArray || arg.prototype instanceof Uint8ClampedArray ||
        arg === Uint16Array || arg.prototype instanceof Uint16Array ||
        arg === Uint32Array || arg.prototype instanceof Uint32Array ||
        arg === Int8Array || arg.prototype instanceof Int8Array ||
        arg === Int16Array || arg.prototype instanceof Int16Array ||
        arg === Int32Array || arg.prototype instanceof Int32Array ||
        arg === Float32Array || arg.prototype instanceof Float32Array ||
        arg === Float64Array || arg.prototype instanceof Float64Array ||
        arg === ArrayBuffer || arg.prototype instanceof ArrayBuffer ||
        arg === DataView || arg.prototype instanceof DataView;
}
//# sourceMappingURL=attribute.js.map