"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unmarshallItem = void 0;
const protocols_1 = require("./protocols");
const value_constructor_exception_1 = require("./value-constructor-exception");
function unmarshallItem(input, valueConstructor) {
    let unmarshalled;
    try {
        unmarshalled = new valueConstructor();
    }
    catch (e) {
        throw new value_constructor_exception_1.ValueConstructorException(`Error to convert input: ${JSON.stringify(input)} and valueConstructor: ${JSON.stringify(valueConstructor)}`);
    }
    const schema = unmarshalled[protocols_1.DocumentSchema];
    for (const key of Object.keys(schema)) {
        const { attributeName = key } = schema[key];
        if (attributeName in input) {
            unmarshalled[key] = unmarshallValue(schema[key], input[attributeName]);
        }
    }
    return unmarshalled;
}
exports.unmarshallItem = unmarshallItem;
function unmarshallValue(schemaType, input) {
    switch (schemaType.type) {
        case 'String':
            return input;
        case 'Boolean':
            if (input === 1) {
                return true;
            }
            else if (input === 0) {
                return false;
            }
            return input;
        case 'Document':
            return input ? unmarshallItem(input, schemaType.valueConstructor) : undefined;
        case 'Collection':
            return input ? input.map(item => {
                if (schemaType.documentConstructorMapping) {
                    const attributeValue = item[schemaType.documentConstructorMapping.attribute];
                    const valueConstructor = schemaType.documentConstructorMapping.mapping[attributeValue];
                    if (valueConstructor) {
                        return unmarshallItem(item, valueConstructor);
                    }
                }
                return unmarshallItem(item, schemaType.valueConstructor);
            }) : undefined;
        case 'Set':
            switch (schemaType.memberType) {
                case 'Number':
                    return unmarshallNumberSet(input);
                case 'String':
                    return unmarshallStringSet(input);
            }
    }
    return input;
}
function unmarshallNumberSet(input) {
    if (!input.values) {
        return new Set();
    }
    const set = new Set();
    input.values.forEach(item => set.add(item));
    return set;
}
function unmarshallStringSet(input) {
    if (!input.values) {
        return new Set();
    }
    const set = new Set();
    input.values.forEach(item => set.add(item));
    return set;
}
//# sourceMappingURL=unmarshall.js.map